@@ -55,45 +55,41 @@ midiprobe(parent, cf, aux)
 	void *aux;
 {
 	register struct isa_attach_args *ia = (struct isa_attach_args *)aux;
-	struct midi_softc softc;
+	struct midi_iface *iface;
+	struct midi_isa_iface loc, *new_loc;
+	int s;
 
-	/*
-	 * We're faking a temp softc here.  All it really needs is
-	 * the addr field, but we supply it so midiforceintr can
-	 * call midi_reset
-	 */
-	softc.addr = ia->ia_iobase;
+	loc.io_port = ia->ia_iobase;
+	if (ia->ia_irq == IRQUNK)
+		loc.irq = -1;
+	else
+		loc.irq = ffs(ia->ia_irq);
 
+	iface = midi_dev_probe(NULL, cf->cf_flags, MIDI_ISA_IFACE, &loc);
+	if (iface == NULL)
+		return (0);
+
+	new_loc = (struct midi_isa_iface *)iface->loc;
 	/* should be able to do a reset if it exists */
-	if (ia->ia_irq == IRQUNK) {
-		ia->ia_irq = isa_discoverintr(midiforceintr, &softc);
+	if (new_loc->irq != -1)
+		ia->ia_irq = (1 << new_loc->irq);
+	else {
+		ia->ia_irq = isa_discoverintr(iface->gen_intr, iface);
 		if (ffs(ia->ia_irq) - 1 == 0)
 			return (0);
 	}
-	if (!midi_reset(&softc))
-		return (0);
 
-	/* We only use addr and addr + 1 */
-	ia->ia_iosize = 2;
+	iface->size(iface, &s);
+	ia->ia_iosize = s;
 	ia->ia_msize = 0;
 
-	return (1);
-}
-
-void
-midiforceintr(softc)
-	struct midi_softc *softc;
-{
-
 	/*
-	 * If the first reset fails, it's because we are in UART
-	 * mode.  UART mode does not generate interrupts or
-	 * acks for commands.  Thus the first reset takes the
-	 * board out of UART mode and the second will cause
-	 * an ack and an interrupt.
+	 * we'll do this for real in attach when we have a real softc
+	 * to put it in.
 	 */
-	if (!midi_reset(softc))
-		midi_reset(softc);
+	iface->free(iface);
+
+	return (1);
 }
 
 void
@@ -104,15 +100,19 @@ midiattach(parent, self, aux)
 	register struct isa_attach_args *ia = (struct isa_attach_args *)aux;
 	struct bsdi20_midi_softc *bsdi_softc;
 	struct midi_softc *softc;
-	u_char rev;
+	struct midi_isa_iface loc;
 
 	bsdi_softc = (struct bsdi20_midi_softc *)self;
 	softc = &bsdi_softc->s;
 
-	rev = midi_init_dev(softc, ia->ia_iobase);
-	if (rev & 0x04)
-		printf(" SMPTE Equipped");
-	printf("\n");
+	loc.io_port = ia->ia_iobase;
+	loc.irq = ffs(ia->ia_irq);
+
+	if (!midi_init_dev(softc, bsdi_softc->sc_dev.dv_flags, MIDI_ISA_IFACE,
+	    &loc))
+		return;
+
+	printf(" %s\n", softc->iface->name(softc->iface));
 
 	/* allocate memory for event queues */
 	if ((softc->rqueue = malloc(sizeof(struct event_queue), M_DEVBUF,
@@ -128,9 +128,18 @@ midiattach(parent, self, aux)
 		free(softc->rqueue, M_DEVBUF);
 		return;
 	}
+	if ((softc->thruqueue = malloc(sizeof(struct event_queue), M_DEVBUF,
+	    M_NOWAIT)) == NULL) {
+		log(LOG_ERR, "No memory for thruqueue\n");
+		softc->status |= MIDI_UNAVAILABLE;
+		free(softc->rqueue, M_DEVBUF);
+		free(softc->wqueue, M_DEVBUF);
+		return;
+	}
 	/* zero read/write queue to clear stynamic structures */
 	bzero(softc->rqueue, sizeof(struct event_queue));
 	bzero(softc->wqueue, sizeof(struct event_queue));
+	bzero(softc->thruqueue, sizeof(struct event_queue));
 	stynamic_init(&softc->rpartial);
 	stynamic_init(&softc->wpartial);
 	softc->wpartialpos = 0;
@@ -235,12 +244,15 @@ midiioctl(dev, cmd, data, flag, p)
 	int flag;
 	struct proc *p;
 {
-	int unit;
+	int f, unit;
 	struct bsdi20_midi_softc *sc;
 
+	f = 0;
+	if (minor(dev) & 0x40)
+		f |= MIDI_RAW_DEVICE;
 	unit = minor(dev) & 0x3f;
 	sc = midicd.cd_devs[unit];
-	return (gen_midiioctl(&sc->s, cmd, data));
+	return (gen_midiioctl(&sc->s, cmd, data, f));
 }
 
 /*
@@ -283,9 +295,14 @@ midiselect(dev, which, p)
 		}
 		break;
 	default:
-		ret = 0;
-		softc->status |= MIDI_ESEL;
-		selrecord(p, &softc->esel);
+		if (softc->status & MIDI_TIME_WARP) {
+			softc->status &= ~MIDI_TIME_WARP;
+			ret = 1;
+		} else {
+			ret = 0;
+			softc->status |= MIDI_ESEL;
+			selrecord(p, &softc->esel);
+		}
 		break;
 	}
 	splx(s);
@@ -326,10 +343,23 @@ void
 TIMEOUT(id, fn, arg, t)
 	TIMEOUT_ID *id;
 	TIMEOUT_FN fn;
-	void *arg;
+	TIMEOUT_ARG arg;
 	int t;
 {
 
 	timeout(fn, arg, t);
 	*id = fn;
 }
+
+int
+FIND_IRQ(irqmask)
+	int irqmask;
+{
+	int found_mask;
+
+	found_mask = isa_irqalloc(irqmask);
+	if (found_mask == 0)
+		return (-1);
+	else
+		return (ffs(found_mask));
+}
