@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -87,6 +87,8 @@ int midiintr __P((struct bsdi20_midi_softc *softc));
  */
 #define OUTB(x, y)	outb((x), (y))
 #define INB(x)		inb((x))
+#define OUTW(x, y)	outw((x), (y))
+#define INW(x)		inw((x))
 #define LOGERR(x)	log(LOG_ERR, x)
 #define LOGWARN(x)	log(LOG_WARNING, x)
 #define PRINTF(x)	printf(x)
@@ -100,10 +102,13 @@ int midiintr __P((struct bsdi20_midi_softc *softc));
 #define UIO_REMAIN(x)		((struct uio *)(x))->uio_resid
 #define MOVE_TO_UIO(uio, src, num)	uiomove((src), (num), \
 					    (struct uio *)(uio))
-#define IOCTL_U_TO_K(dest, src, sz)	bcopy(src, dest, sz)
-#define IOCTL_K_TO_U(dest, src, sz)	bcopy(src, dest, sz)
+#define IOCTL_U_TO_K(dest, src, sz)	bcopy((src), (dest), (sz))
+#define IOCTL_K_TO_U(dest, src, sz)	bcopy((src), (dest), (sz))
+#define FEATURE_U_TO_K(dest, src, sz)	copyin((src), (dest), (sz))
+#define FEATURE_K_TO_U(dest, src, sz)	copyout((src), (dest), (sz))
+#define INIT_TIMEOUT_ID(x)
 #define KERNEL_HZ	hz
-#define BLOCK_INTR	spltty
+#define BLOCK_INTR	splhigh
 #define UNBLOCK_INTR(x)	splx(x)
 #define BZERO(dest, size)	bzero(dest, size)
 #define BCOPY(src, dest, size)	bcopy(src, dest, size)
@@ -122,4 +127,5 @@ int midiintr __P((struct bsdi20_midi_softc *softc));
  */
 extern struct midi_softc *UNIT_TO_SOFTC __P((int unit, void *client));
 extern void SIGNAL_PG __P((int pgid, int sig));
-extern void TIMEOUT __P((TIMEOUT_ID *id, TIMEOUT_FN fn, void *arg, int t));
+extern void TIMEOUT __P((TIMEOUT_ID *id, TIMEOUT_FN fn, TIMEOUT_ARG arg,
+    int t));
