@@ -31,30 +31,43 @@
 /*
  * Linux driver dependent code
  */
+
 #include "midi-linux.h"
 #include "midifn.h"
 
-#ifdef MODULE
-#include "release.h"
+#include <linux/module.h>
+
+#ifdef NEED_KERNEL_VERSION_STRING
+char kernel_version[] = UTS_RELEASE;
 #endif
 
 /*
  * This is where you configure the MPU401 devices.
  * Define NMIDI to be the number of MPU401 devices you have installed.
  * And add an entry to the midi_conf array describing each
- * device's I/O addr and IRQ number.
+ * device's type, I/O addr and IRQ number.
+ * For a list of types and their codes, see the MIDI_DEV_* defines
+ * in midi-iface.h.
  */
 
 #define NMIDI 1
 int NumMidi = NMIDI;
 struct midi_config midi_conf[NMIDI] = {
-	{0x330, 5}			/* 'standard' MPU401 */
+	{MIDI_DEV_MPU401, 0x330, 5}	/* 'standard' MPU401 */
 };
 struct linux_midi_softc midi_sc[NMIDI];
 
 
 extern volatile u_long jiffies;
+
+/*
+ * SMP support broke this in 1.3.31.. current is now a macro
+ * Seems like an awfully generic name to make into a macro..
+ * My guess is that it will change again soon..
+ */
+#if (LINUX_VERSION_CODE < 0x01031F)
 extern struct task_struct *current;
+#endif
 
 static struct file_operations midi_fops = {
 	NULL,		/* lseek */
@@ -86,6 +99,7 @@ init_module(void)
 void
 cleanup_module(void)
 {
+	struct midi_isa_iface *loc;
 	struct midi_softc *softc;
 	int i;
 
@@ -105,7 +119,13 @@ cleanup_module(void)
 		midi_fullreset(softc);
 		kfree_s(softc->wqueue, sizeof(struct event_queue));
 		kfree_s(softc->rqueue, sizeof(struct event_queue));
-		free_irq(softc->intr);
+		kfree_s(softc->thruqueue, sizeof(struct event_queue));
+		loc = (struct midi_isa_iface *)softc->iface->loc;
+		free_irq(loc->irq);
+		softc->iface->free(softc->iface);
+		softc->timer->release_id(softc->timeout_id);
+		softc->timer->release_id(softc->thru_timeout_id);
+		softc->timer->free(softc->timer);
 	}
 	if (unregister_chrdev(MIDI_MAJOR, "midi") != 0)
 		printk("cleanup_module failed\n");
@@ -118,9 +138,9 @@ long
 midiinit(kmem_start)
 	long kmem_start;
 {
+	struct midi_isa_iface loc, *ploc;
 	struct midi_softc *softc;
 	int i;
-	int rev;
 
 	if (register_chrdev(MIDI_MAJOR, "midi", &midi_fops)) {
 		printk("unable to get major %d for MPU401 MIDI\n",
@@ -129,22 +149,27 @@ midiinit(kmem_start)
 	}
 	for (i = 0; i < NumMidi; i++) {
 		softc = &(midi_sc[i].s);
-		softc->intr = midi_conf[i].intr;
-		init_timer(&softc->timeout_id);
-		init_timer(&softc->smpte_timeout_id);
-		init_timer(&softc->write_timeout_id);
-		rev = midi_init_dev(softc, midi_conf[i].addr);
-		if (rev == -1) {
-			printk("Couldn't init MIDI dev at 0x%x, IRQ %x\n",
-			    midi_conf[i].addr, softc->intr);
+
+		loc.irq = midi_conf[i].intr;
+		loc.io_port = midi_conf[i].addr;
+		if (!midi_init_dev(softc, midi_conf[i].type, MIDI_ISA_IFACE,
+		    &loc)) {
+			printk("Couldn't init MIDI dev %d at 0x%x, IRQ %x\n",
+			    (int)midi_conf[i].type, loc.io_port, loc.irq);
 			continue;
 		}
+
+		init_timer(softc->timeout_id);
+		init_timer(softc->thru_timeout_id);
+
 		/* allocate memory for event queues */
 #ifndef MODULE
 		softc->rqueue = (struct event_queue *)kmem_start;
 		kmem_start += sizeof(struct event_queue);
 		softc->wqueue = (struct event_queue *)kmeme_start;
 		kmem_start += sizeof(struct event_queue);
+		softc->thruqueue = (struct event_queue *)kmeme_start;
+		kmem_start += sizeof(struct event_queue);
 #else
 		softc->rqueue = kmalloc(sizeof(struct event_queue),
 		    GFP_KERNEL);
@@ -159,32 +184,41 @@ midiinit(kmem_start)
 			kfree_s(softc->rqueue, sizeof(struct event_queue));
 			return (-1);
 		}
+		softc->thruqueue = kmalloc(sizeof(struct event_queue),
+		    GFP_KERNEL);
+		if (softc->thruqueue == 0) {
+			printk("Out of memory for thru queue\n");
+			kfree_s(softc->rqueue, sizeof(struct event_queue));
+			kfree_s(softc->wqueue, sizeof(struct event_queue));
+			return (-1);
+		}
 #endif
 		memset(softc->rqueue, 0, sizeof(struct event_queue));
 		memset(softc->wqueue, 0, sizeof(struct event_queue));
+		memset(softc->thruqueue, 0, sizeof(struct event_queue));
 		stynamic_init(&softc->rpartial);
 		stynamic_init(&softc->wpartial);
 		softc->wpartialpos = 0;
 
 		/* register the intr */
-		if (request_irq(softc->intr, (void *)midiintr, SA_INTERRUPT,
+		ploc = (struct midi_isa_iface *)softc->iface->loc;
+		if (request_irq(ploc->irq, (void *)midiintr, SA_INTERRUPT,
 		    "midi")) {
-			printk("MPU401 #%d unable to use interrupt %d\n",
-			    i, softc->intr);
+			printk("MIDI #%d unable to use interrupt %d\n",
+			    i, ploc->irq);
 #ifndef MODULE
 			kmem_start -= 2 * sizeof(struct event_queue);
 #else
 			kfree_s(softc->rqueue, sizeof(struct event_queue));
 			kfree_s(softc->wqueue, sizeof(struct event_queue));
+			kfree_s(softc->thruqueue, sizeof(struct event_queue));
 #endif
 			softc->status |= MIDI_UNAVAILABLE;
 			continue;
 		}
-		printk("Found MPU401 #%d at 0x%02x irq %d", i, softc->addr,
-		    softc->intr);
-		if (softc->features & SMPTE_EQUIP)
-			printk(" SMPTE Equipped");
-		printk("\n");
+		printk("Found %s #%d at 0x%02x irq %d\n",
+		    softc->iface->name(softc->iface), i, ploc->io_port,
+		    ploc->irq);
 	}
 	return(kmem_start);
 }
@@ -313,11 +347,16 @@ midiintr(irq, reg)
 	struct pt_regs *reg;
 {
 	struct midi_softc *softc;
+	struct midi_isa_iface *loc;
 	int i;
 
-	for (i = 0; i < NumMidi && midi_sc[i].s.intr != irq; i++);
+	for (i = 0; i < NumMidi; i++) {
+		loc = (struct midi_isa_iface *)midi_sc[i].s.iface->loc;
+		if (loc->irq == irq)
+			break;
+	}
 	if (i == NumMidi) {
-		printk("midi: ivec(%d) is not a valid MPU401 intr number\n",
+		printk("midi: ivec(%d) is not a valid MIDI intr number\n",
 		    irq);
 		return;
 	}
@@ -334,10 +373,14 @@ midiioctl(inode, file, cmd, arg)
 	unsigned long arg;
 {
 	struct midi_softc *softc;
+	int f;
 
+	f = 0;
+	if (MINOR(inode->i_rdev) & 0x40)
+		f |= MIDI_RAW_DEVICE;
 	softc = &(midi_sc[MINOR(inode->i_rdev) & 0x3f].s);
 
-	return (-gen_midiioctl(softc, cmd, (void *)arg));
+	return (-gen_midiioctl(softc, cmd, (void *)arg, f));
 }
 
 /*
@@ -376,9 +419,14 @@ midiselect(inode, file, type, wait)
 		}
 		break;
 	case SEL_EX:
-		softc->status |= MIDI_ESEL;
-		select_wait(&softc->esel, wait);
-		return (0);
+		if (softc->status & MIDI_TIME_WARP) {
+			softc->status &= ~MIDI_TIME_WARP;
+			return (1);
+		} else {
+			softc->status |= MIDI_ESEL;
+			select_wait(&softc->esel, wait);
+			return (0);
+		}
 	default:
 		return (0);
 	}
@@ -468,7 +516,7 @@ void
 TIMEOUT(id, fn, arg, t)
 	TIMEOUT_ID *id;
 	TIMEOUT_FN fn;
-	void *arg;
+	TIMEOUT_ARG arg;
 	int t;
 {
 
@@ -478,7 +526,7 @@ TIMEOUT(id, fn, arg, t)
 #else
 	id->expires = jiffies + t;
 #endif
-	id->data = (TIMEOUT_ARG)arg;
+	id->data = arg;
 	id->function = fn;
 	add_timer(id);
 }
