@@ -99,6 +99,11 @@ midi_unload(void)
 		midi_fullreset(softc);
 		kmem_free(softc->rqueue, sizeof(struct event_queue));
 		kmem_free(softc->wqueue, sizeof(struct event_queue));
+		kmem_free(softc->thruqueue, sizeof(struct event_queue));
+		softc->iface->free(softc->iface);
+		softc->timer->release_id(softc->timeout_id);
+		softc->timer->release_id(softc->thru_timeout_id);
+		softc->timer->free(softc->timer);
 	}
 	mod_drvdetach(&midi_attach_info);
 	return 0;
@@ -110,15 +115,18 @@ midiinit()
 {
 	struct unixware_midi_softc *usc;
 	struct midi_softc *softc;
+	struct midi_isa_iface loc, *ploc;
 	int i;
 
 	for (i = 0; i < NumMidi; i++) {
 		usc = &midi_sc[i];
 		softc = &usc->s;
-		softc->addr = midi_addrs[i];
-		softc->intr = midi_intrs[i];
 
-		if (midi_init_dev(softc, softc->addr) == -1)
+		loc.irq = midi_intrs[i];
+		loc.io_port = midi_addrs[i];
+
+		/* XXX!!! Got to get a flags field from somewhere */
+		if (!midi_init_dev(softc, midi_types[i], MIDI_ISA_IFACE, &loc))
 			continue;
 		if ((softc->rqueue = kmem_alloc(sizeof(struct event_queue),
 		    KM_NOSLEEP)) == NULL) {
@@ -133,17 +141,25 @@ midiinit()
 			kmem_free(softc->rqueue, sizeof(struct event_queue));
 			continue;
 		}
+		if ((softc->thruqueue = kmem_alloc(sizeof(struct event_queue),
+		    KM_NOSLEEP)) == NULL) {
+			printf("No memory for thruqueue\n");
+			softc->status |= MIDI_UNAVAILABLE;
+			kmem_free(softc->rqueue, sizeof(struct event_queue));
+			kmem_free(softc->wqueue, sizeof(struct event_queue));
+			continue;
+		}
 		/* zero read/write queue to clear stynamic structures */
 		bzero((caddr_t)softc->rqueue, sizeof(struct event_queue));
 		bzero((caddr_t)softc->wqueue, sizeof(struct event_queue));
+		bzero((caddr_t)softc->thruqueue, sizeof(struct event_queue));
 		stynamic_init(&softc->rpartial);
 		stynamic_init(&softc->wpartial);
 		softc->wpartialpos = 0;
-		printf("Found MPU401 #%d at 0x%02x irq %d", i, softc->addr,
-		    softc->intr);
-		if (softc->features & SMPTE_EQUIP)
-			printf(" SMPTE Equipped");
-		printf("\n");
+		ploc = (struct midi_isa_iface *)softc->iface->loc;
+		printf("Found %s #%d at 0x%02x irq %d\n",
+		    softc->iface->name(softc->iface), i, ploc->io_port,
+		    ploc->irq);
 	}
 }
 
@@ -238,12 +254,16 @@ midiintr(ivec)
 	int ivec;
 {
 	struct midi_softc *softc;
+	struct midi_isa_iface *loc;
 	int i;
 
-	for (i = 0; i < NumMidi && midi_sc[i].s.intr != ivec; i++)
-		;
+	for (i = 0; i < NumMidi; i++) {
+		loc = (struct midi_isa_iface *)midi_sc[i].s.iface->loc;
+		if (loc->irq == ivec)
+			break;
+	}
 	if (i == NumMidi) {
-		printf("midi: ivec(%d) is not a valid MPU401 intr number\n",
+		printf("midi: ivec(%d) is not a valid MIDI intr number\n",
 		    ivec);
 		return (0);
 	}
@@ -261,10 +281,14 @@ midiioctl(dev, cmd, arg, mode, cred_p, rval_p)
 	int *rval_p;
 {
 	struct midi_softc *softc;
+	int f;
 
+	f = 0;
+	if (getminor(dev) & 0x40)
+		f |= MIDI_RAW_DEVICE;
 	softc = &(midi_sc[getminor(dev) & 0x3f].s);
 
-	return (gen_midiioctl(softc, cmd, &arg));
+	return (gen_midiioctl(softc, cmd, &arg, f));
 }
 
 /*
@@ -291,13 +315,12 @@ midichpoll(dev, events, anyyet, reventsp, phpp)
 		*reventsp |= POLLIN;
 	if (events & POLLOUT && !(softc->status & MIDI_WR_BLOCK))
 		*reventsp |= POLLOUT;
-	/* I don't think POLLPRI will ever work since it is a transient event */
-	/* can't statisfy POLLPRI immediately, wait for timewarp */
 
-#if 0 	/* to avoid a warning */
-	if (events & POLLPRI)
-		;
-#endif
+	/* to avoid a warning */
+	if (events & POLLPRI && (softc->status & MIDI_TIME_WARP)) {
+		softc->status &= ~MIDI_TIME_WARP;
+		*reventsp |= POLLPRI;
+	}
 	if (*reventsp == 0)
 		if (!anyyet)
 			*phpp = &softc->rsel;
@@ -333,9 +356,9 @@ void
 TIMEOUT(id, fn, arg, t)
 	TIMEOUT_ID *id;
 	TIMEOUT_FN fn;
-	void *arg;
+	TIMEOUT_ARG arg;
 	int t;
 {
 
-	*id = timeout(fn, (TIMEOUT_ARG)arg, t);
+	*id = timeout(fn, arg, t);
 }
