@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,19 +28,18 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-/*
- * midi port offsets
- */
-#define	MIDI_DATA			0
-#define	MIDI_STATUS			1
-#define	MIDI_COMMAND			1
+#ifndef MIDIVAR_H
+#define MIDIVAR_H
 
+#include "midi-event.h"
+#include "midi-timer.h"
 
 /*
- * midi data transfer status bits
+ * This is the maximum number of midi devices we can have in one system.
+ * This is a software limit, not a hardware limit.  If you want more,
+ * just increase this value.
  */
-#define	MIDI_RDY_RCV			(1 << 6)
-#define	MIDI_DATA_AVL			(1 << 7)
+#define MAX_MIDI_DEV 4
 
 /*
  * midi status flags
@@ -55,22 +54,21 @@
 #define MIDI_FLUSH_SLEEP	(1 << 7) /* blocking on flush */
 #define MIDI_RD_SLEEP		(1 << 8) /* blocking on read */
 #define MIDI_WR_SLEEP		(1 << 9) /* blocking on write */
-#define MIDI_BUFFER_SLEEP	(1 << 10) /* blocking on buffer */
-#define MIDI_NEED_ACK		(1 << 11) /* command needs and ack */
-#define MIDI_ASYNC		(1 << 12) /* send sigios */
-#define MIDI_SENDIO		(1 << 13) /* a sigio should be send at low h2o */
-#define MIDI_THRU		(1 << 14) /* pass in port to out port? */
-#define MIDI_RECONPLAY		(1 << 15) /* don't record until we start to play */
-#define MIDI_EXTCLK		(1 << 16) /* use external clock */
-#define MIDI_NEED_DATA		(1 << 17) /* need to read command response */
-#define MIDI_FIRST_WRITE	(1 << 18) /* the next write will be the first */
-#define MIDI_FIRST_SMPTE	(1 << 19) /* set if no SMPTE yet */
-#define MIDI_RSEL		(1 << 20) /* select on read active */
-#define MIDI_WSEL		(1 << 21) /* select on write active */
-#define MIDI_ESEL		(1 << 22) /* select on execption active */
-#define MIDI_TIMEOUT_PENDING	(1 << 23) /* timeout is pending */
-#define MIDI_WRITING_EVENT	(1 << 24) /* protect writing an event */
-#define MIDI_WE_SLEEP		(1 << 25) /* waiting to write an event */
+#define MIDI_ASYNC		(1 << 11) /* send sigios */
+#define MIDI_SENDIO		(1 << 12) /* a sigio should be send at low h2o */
+#define MIDI_THRU		(1 << 13) /* pass in port to out port? */
+#define MIDI_RECONPLAY		(1 << 14) /* don't record until we start to play */
+#define MIDI_EXTCLK		(1 << 15) /* use external clock */
+#define MIDI_FIRST_WRITE	(1 << 16) /* reset the clock on first t/o */
+#define MIDI_FIRST_SMPTE	(1 << 17) /* set if no SMPTE yet */
+#define MIDI_RSEL		(1 << 18) /* select on read active */
+#define MIDI_WSEL		(1 << 19) /* select on write active */
+#define MIDI_ESEL		(1 << 20) /* select on execption active */
+#define MIDI_TIMEOUT_PENDING	(1 << 21) /* timeout is pending */
+#define MIDI_THRU_CB_SCHED	(1 << 24) /* thru callback scheduled? */
+#define MIDI_MASTER		(1 << 25) /* are we a master? */
+#define MIDI_TIME_WARP		(1 << 26) /* did time shift? */
+#define MIDI_WRITING_EVENT	(1 << 27) /* currently writing an event */
 
 /*
  * access flags
@@ -86,36 +84,12 @@
 typedef enum {START, NEEDDATA1, NEEDDATA2, SYSEX, SYSTEM1, SYSTEM2, MTC}
     InputState;
 
-/*
- * midi command values
- */
-#define MIDI_RESET			0xff
-#define MIDI_UART			0x3f
-#define MIDI_ACK			0xfe
-#define MIDI_VERSION			0xac
-#define MIDI_REVISION			0xad
-#define MIDI_TRIES			200000
-
 /*
  * SMPTE configuration
  */
 #define SMPTE_DROPCHK 10	/* check for loss of sync every 10 ticks */
 #define SMPTE_DROPOUT 50	/* 50 ticks without 0xf1 ==> loss of sync */
 
-/*
- * most events are short, so we use the static array,
- * but occationally we get a long event (sysex) and
- * we dynamically allocate that
- */
-#define STYNAMIC_SIZE 4
-#define STYNAMIC_ALLOC 256
-
-struct stynamic {
-	short	allocated;
-	short	len;
-	u_char	datas[4];
-	u_char	*datad;
-};
 
 /*
  * data from the board that hasn't formed a complete event yet
@@ -138,27 +112,6 @@ struct tempo_change {
 	struct	tempo_change *next;
 };
 
-/*
- * the internal representation of an event
- */
-typedef enum {NORMAL, TEMPO, TIMESIG, SYSX, SMPTE, NOP} EventType;
-
-struct event {
-	u_long	time;		/* time until event in kernel clicks */
-	u_long	smf_time;	/*
-				 * absolute time of the event in smf ticks
-				 * only used in writing
-				 */
-	long	tempo;		/*
-				 * not used in play events, but contains
-				 * the tempo setting current when the
-				 * incoming event arrived.  Used for
-				 * converting timing ticks to smf ticks
-				 */
-	EventType	type;
-	struct	stynamic data;
-};
-
 /*
  * A event queue, used for both incoming and outgoing
  */
@@ -186,9 +139,5 @@ struct event_queue {
  * playing any note on events.
  */
 #define BACKLOG_TIME 5
-/*
- * different MPU401 features
- */
-#define SMPTE_EQUIP	(1 << 0)	/* equipped with MQ SMPTE */
-
 
+#endif
