@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -71,7 +71,6 @@ Event *
 EventTree::GetEvents(unsigned long time)
 {
 	EventTreeNode *n;
-	Event *e;
 	int valid_event;
 
 	n = rb_find_ukey(head, time);
@@ -89,7 +88,7 @@ EventTree::GetEvents(unsigned long time)
 	valid_event = 0;
 	do {
 		/* check event type(s) */
-		for (e = curr_event; e != 0; e = e->GetNextEvent()) {
+		while (curr_event != 0) {
 			/*
 			 * It is a valid event if the following
 			 * conditions do *not* occur.
@@ -97,13 +96,14 @@ EventTree::GetEvents(unsigned long time)
 			 * NoteOn event with a velocity of 0,
 			 * and it has a NotePair.
 			 */
-			if (!((e->GetType() == NOTEOFF ||
-			    (e->GetType() == NOTEON &&
-			    ((NoteEvent *)e)->GetVelocity() == 0)) &&
-			    ((NoteEvent *)e)->GetNotePair() != 0)) {
+			if (!((curr_event->GetType() == NOTEOFF ||
+			    (curr_event->GetType() == NOTEON &&
+			    ((NoteEvent *)curr_event)->GetVelocity() == 0)) &&
+			    ((NoteEvent *)curr_event)->GetNotePair() != 0)) {
 				valid_event = 1;
 				break;
 			}
+			curr_event = curr_event->GetNextEvent();
 		}
 		/*
 		 * get the next node
