@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -456,7 +456,7 @@ Song::SMFRead(int fd)
 		tracks[i] = new EventTree;
 		assert(tracks[i] != 0);
 		tracks[i]->Lock();
-		while ((event = ReadEventFromSMFTrack(track, last_t, errstr))
+		while ((event = ReadEventFromSMFTrack(track, last_t, 1, errstr))
 		    != 0) {
 			Event *event_ptr;
 
@@ -501,7 +501,7 @@ Song::SMFWrite(int fd)
 		last_t = 0;
 		for (e = tracks[i]->GetFirstEvent(); e != 0;
 		    e = tracks[i]->NextEvent(e))
-			if (!WriteEventToSMFTrack(track, last_t, e, errstr))
+			if (!WriteEventToSMFTrack(track, last_t, e, 1, errstr))
 				return (0);
 		if (!track.Write(fd))
 			return (0);
