@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,23 +40,6 @@ extern "C" {
 #include "tclmEvnt.h"
 #include "patchlvl.h"
 
-#ifdef _WINDOWS
-extern "C" Tcl_CmdProc __loadds Tclm_MidiMake;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiFree;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiRead;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiWrite;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiConfig;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiRewind;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiGet;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiPut;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiDelete;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiMerge;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiSplit;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiCopy;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiVersion;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiTrack;
-#else
-
 static int Tclm_MidiMake(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv);
 static int Tclm_MidiFree(ClientData client_data, Tcl_Interp *interp, int argc,
@@ -85,7 +68,6 @@ static int Tclm_MidiVersion(ClientData client_data, Tcl_Interp *interp,
     int argc, char **argv);
 static int Tclm_MidiTrack(ClientData client_data, Tcl_Interp *interp,
     int argc, char **argv);
-#endif
 static int Tclm_GetTrack(TclmInterp *tclm_interp, Tcl_Interp *interp,
     const char *str, Song **song, int *track);
 
@@ -120,9 +102,6 @@ Tclm_Init(Tcl_Interp *interp)
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiMake(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -145,9 +124,6 @@ Tclm_MidiMake(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiFree(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -168,9 +144,6 @@ Tclm_MidiFree(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiRead(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -203,9 +176,6 @@ Tclm_MidiRead(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiWrite(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -237,9 +207,6 @@ Tclm_MidiWrite(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiConfig(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -340,9 +307,6 @@ Tclm_MidiConfig(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiRewind(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -373,9 +337,6 @@ Tclm_MidiRewind(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiGet(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -473,9 +434,6 @@ Tclm_MidiGet(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiPut(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -543,9 +501,6 @@ Tclm_MidiPut(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiDelete(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -644,9 +599,6 @@ Tclm_MidiDelete(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiMerge(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -680,9 +632,6 @@ Tclm_MidiMerge(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiSplit(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -718,9 +667,6 @@ Tclm_MidiSplit(ClientData client_data, Tcl_Interp *interp, int argc,
 
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiCopy(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -776,12 +722,13 @@ Tclm_MidiCopy(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiVersion(ClientData client_data, Tcl_Interp *interp, int argc,
     char *argv[])
 {
+	ClientData *dummy;
+
+	// shut up a warning
+	dummy = &client_data;
 
 	if (argc != 1) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -793,9 +740,6 @@ Tclm_MidiVersion(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiTrack(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
