@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -334,6 +334,10 @@ char *
 Tclm_PrintMetaEndOfTrack(MetaEndOfTrackEvent *e)
 {
 	ostrstream buf;
+	MetaEndOfTrackEvent *dummy;
+
+	// shut up a warning
+	dummy = e;
 
 	buf << "MetaEndOfTrack" << ends;
 	return (buf.str());
@@ -926,6 +930,10 @@ Tclm_ParseMetaPortNumber(Tcl_Interp *interp, long time, int argc, char **argv)
 Event *
 Tclm_ParseMetaEndOfTrack(Tcl_Interp *interp, long time, int argc, char **argv)
 {
+	char *dummy;
+
+	// shut up a warning
+	dummy = argv[0];
 
 	if (argc != 1) {
 		Tcl_SetResult(interp, "bad event: should be "
