@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
+ * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -37,22 +37,18 @@ extern "C" {
 #include <ctype.h>
 #include "tclmidi.h"
 #include "TclmIntp.h"
+#include "tclmEvnt.h"
 #include "Song.h"
-#ifdef USE_MPU401
-#include "MPU401.h"
-#endif
-#ifdef USE_MPU401THREAD
-#include "MPU401Thread.h"
+#ifdef USE_TCLMDRIVER
+#include "TclmDr.h"
 #endif
 
-#ifdef _WINDOWS
-extern "C" Tcl_CmdProc __loadds Tclm_MidiPlay;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiRecord;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiStop;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiWait;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiDevice;
-extern "C" Tcl_CmdProc __loadds Tclm_MidiTime;
+#if defined(USE_TCLMDRIVER)
+static const int DeviceConfigured = 1;
 #else
+static const int DeviceConfigured = 0;
+#endif
+
 static int Tclm_MidiPlay(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv);
 static int Tclm_MidiRecord(ClientData client_data, Tcl_Interp *interp, int argc,
@@ -65,21 +61,17 @@ static int Tclm_MidiDevice(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv);
 static int Tclm_MidiTime(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv);
-#endif
-static void GetTime(ostrstream &buf, MidiDevice *dev, TimeType type);
+static int Tclm_MidiFeature(ClientData client_data, Tcl_Interp *interp,
+    int argc, char **argv);
+static int Tclm_MidiSend(ClientData client_data, Tcl_Interp *interp, int argc,
+    char **argv);
+static int Tclm_MidiRecv(ClientData client_data, Tcl_Interp *interp, int argc,
+    char **argv);
 
 int
 Tclm_PlayInit(Tcl_Interp *interp, TclmInterp *tclm_interp)
 {
 
-#if defined(USE_MPU401)
-	tclm_interp->SetMidiDevice(new MPU401("/dev/midi0"));
-#elif defined(USE_MPU401THREAD)
-	tclm_interp->SetMidiDevice(new MPU401Thread("/dev/midi0"));
-#else
-	tclm_interp->SetMidiDevice(0);
-#endif
-
 	Tcl_CreateCommand(interp, "midiplay", Tclm_MidiPlay, tclm_interp, 0);
 	Tcl_CreateCommand(interp, "midirecord", Tclm_MidiRecord,
 	    tclm_interp, 0);
@@ -88,13 +80,14 @@ Tclm_PlayInit(Tcl_Interp *interp, TclmInterp *tclm_interp)
 	Tcl_CreateCommand(interp, "mididevice", Tclm_MidiDevice,
 	    tclm_interp, 0);
 	Tcl_CreateCommand(interp, "miditime", Tclm_MidiTime, tclm_interp, 0);
+	Tcl_CreateCommand(interp, "midifeature", Tclm_MidiFeature,
+	    tclm_interp, 0);
+	Tcl_CreateCommand(interp, "midisend", Tclm_MidiSend, tclm_interp, 0);
+	Tcl_CreateCommand(interp, "midirecv", Tclm_MidiRecv, tclm_interp, 0);
 	return (TCL_OK);
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiPlay(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -103,31 +96,31 @@ Tclm_MidiPlay(ClientData client_data, Tcl_Interp *interp, int argc,
 	MidiDevice *dev;
 	int repeat;
 
-	if (argc != 2 && argc != 3) {
+	if (argc != 3 && argc != 4) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " MidiID ?repeat?\"", 0);
+		    argv[0], " DevID MidiID ?repeat?\"", 0);
 		return (TCL_ERROR);
 	}
 	tclm_interp = (TclmInterp *)client_data;
 
-	dev = tclm_interp->GetMidiDevice();
+	dev = tclm_interp->GetDevice(argv[1]);
 	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
 	}
 
-	if ((song = tclm_interp->GetSong(argv[1])) == 0) {
-		Tcl_AppendResult(interp, "bad key ", argv[1], 0);
+	if ((song = tclm_interp->GetSong(argv[2])) == 0) {
+		Tcl_AppendResult(interp, "bad SongID ", argv[2], 0);
 		return (TCL_ERROR);
 	}
 
 	repeat = 0;
-	if (argc == 3 && strlen(argv[2]) != 0) {
-		if (strcmp(argv[2], "repeat") == 0)
+	if (argc == 4 && strlen(argv[3]) != 0) {
+		if (strcmp(argv[3], "repeat") == 0)
 			repeat = 1;
 		else {
 			Tcl_AppendResult(interp, "bad repeat option: should "
-			    "be \"", argv[0], " MidiID ?repeat?\"", 0);
+			    "be \"", argv[0], " DevID MidiID ?repeat?\"", 0);
 			return (TCL_ERROR);
 		}
 	}
@@ -142,9 +135,6 @@ Tclm_MidiPlay(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiRecord(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
@@ -153,38 +143,38 @@ Tclm_MidiRecord(ClientData client_data, Tcl_Interp *interp, int argc,
 	MidiDevice *dev;
 	int repeat;
 
-	if (argc < 2 || argc > 4) {
+	if (argc < 3 || argc > 5) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " RecMidiID ?PlayMidiID ?repeat??\"", 0);
+		    argv[0], " DevID RecMidiID ?PlayMidiID ?repeat??\"", 0);
 		return (TCL_ERROR);
 	}
 	tclm_interp = (TclmInterp *)client_data;
 
-	dev = tclm_interp->GetMidiDevice();
+	dev = tclm_interp->GetDevice(argv[1]);
 	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
 	}
 
-	if ((rsong = tclm_interp->GetSong(argv[1])) == 0) {
-		Tcl_AppendResult(interp, "bad key ", argv[1], 0);
+	if ((rsong = tclm_interp->GetSong(argv[2])) == 0) {
+		Tcl_AppendResult(interp, "bad SongID ", argv[2], 0);
 		return (TCL_ERROR);
 	}
 
 	psong = 0;
 	repeat = 0;
-	if (argc > 2) {
-		if ((psong = tclm_interp->GetSong(argv[2])) == 0) {
-			Tcl_AppendResult(interp, "bad key ", argv[2], 0);
+	if (argc > 3) {
+		if ((psong = tclm_interp->GetSong(argv[3])) == 0) {
+			Tcl_AppendResult(interp, "bad SongID ", argv[3], 0);
 			return (TCL_ERROR);
 		}
 
-		if (argc == 4 && strlen(argv[3]) != 0) {
-			if (strcmp(argv[3], "repeat") == 0)
+		if (argc == 5 && strlen(argv[4]) != 0) {
+			if (strcmp(argv[4], "repeat") == 0)
 				repeat = 1;
 			else {
 				Tcl_AppendResult(interp, "bad repeat flag: ",
-				    argv[3], 0);
+				    argv[4], 0);
 				return (TCL_ERROR);
 			}
 		}
@@ -200,26 +190,23 @@ Tclm_MidiRecord(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiStop(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
 	TclmInterp *tclm_interp;
 	MidiDevice *dev;
 
-	if (argc != 1) {
+	if (argc != 2) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], "\"", 0);
+		    argv[0], " DevID\"", 0);
 		return (TCL_ERROR);
 	}
 	tclm_interp = (TclmInterp *)client_data;
 
-	dev = tclm_interp->GetMidiDevice();
+	dev = tclm_interp->GetDevice(argv[1]);
 	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
 	}
 
 	if (!dev->Stop()) {
@@ -232,25 +219,22 @@ Tclm_MidiStop(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiWait(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
 	TclmInterp *tclm_interp;
 	MidiDevice *dev;
 
-	if (argc != 1) {
+	if (argc != 2) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], 0);
+		    argv[0], " DevID\"", 0);
 		return (TCL_ERROR);
 	}
 	tclm_interp = (TclmInterp *)client_data;
-	dev = tclm_interp->GetMidiDevice();
+	dev = tclm_interp->GetDevice(argv[1]);
 	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
 	}
 	if (!dev->Wait()) {
 		Tcl_AppendResult(interp, "Couldn't wait for playing/recording "
@@ -262,62 +246,83 @@ Tclm_MidiWait(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiDevice(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
 	TclmInterp *tclm_interp;
 	MidiDevice *dev;
 	ostrstream *buf;
-	char *str, **sub_argv;
-	int i, mt, sub_argc, value;
+	unsigned short mask;
+	char *dev_key, *str, **sub_argv;
+	int con, i, imask, j, mt, sub_argc, value;
 
 	tclm_interp = (TclmInterp *)client_data;
-	dev = tclm_interp->GetMidiDevice();
-	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
-	}
 
 	/*
 	 * mididevice with no args means just check for device
 	 * support.
 	 */
 	if (argc == 1) {
-		Tcl_SetResult(interp, "1", TCL_STATIC);
+		buf = new ostrstream;
+		*buf << DeviceConfigured << ends;
+		str = buf->str();
+		Tcl_SetResult(interp, str, TCL_VOLATILE);
+		delete buf;
+		delete str;
 		return (TCL_OK);
 	}
 
-	for (i = 1; i < argc; i++) {
+	// Either open or get device
+	if (strncmp(argv[1], "device", strlen("device")) == 0) {
+		// get an existing device
+		dev = tclm_interp->GetDevice(argv[1]);
+		if (dev == 0) {
+			Tcl_AppendResult(interp, "Bad device id \"",
+			    argv[1], "\"", 0);
+			return (TCL_ERROR);
+		}
+		dev_key = argv[1];
+	} else {
+#ifdef USE_TCLMDRIVER
+		dev = new TclmDriver;
+#else
+		dev = 0;
+#endif
+		if (dev == 0) {
+			Tcl_SetResult(interp, "Out of memory", TCL_STATIC);
+			return (TCL_ERROR);
+		}
+		if (!dev->Open(argv[1])) {
+			Tcl_AppendResult(interp, "Couldn't open ",
+			    argv[1], ": ", dev->GetError(), 0);
+			return (TCL_ERROR);
+		}
+		dev_key = tclm_interp->AddDevice(dev);
+		if (dev_key == 0) {
+			Tcl_SetResult(interp, "Couldn't add device",
+			    TCL_STATIC);
+			return (TCL_ERROR);
+		}
+		Tcl_AppendResult(interp, dev_key, 0);
+	}
+
+	for (i = 2; i < argc; i++) {
 		// loop through each arg and either set or return values
 		if (Tcl_SplitList(interp, argv[i], &sub_argc, &sub_argv)
 		    != TCL_OK)
 			return (TCL_ERROR);
-		if (strcmp(sub_argv[0], "name") == 0) {
-			switch (sub_argc) {
-			case 1:
-				/* return the value */
-				buf = new ostrstream;
-				*buf << "name \"" << dev->GetName() << "\""
-				    << ends;
-				str = buf->str();
-				Tcl_AppendElement(interp, str);
-				delete str;
-				delete buf;
-				break;
-			case 2:
-				/* set the value */
-				dev->SetName(sub_argv[1]);
-				break;
-			default:
-				/* wrong args */
-				Tcl_AppendResult(interp, "Wrong number of "
-				    "args: should be \"", argv[0], " name "
-				    "?DevName?\"", 0);
+		if (strcmp(sub_argv[0], "close") == 0) {
+			if (i != argc - 1) {
+				Tcl_SetResult(interp, "Can't perform ops after "
+				    "closing device", TCL_STATIC);
 				return (TCL_ERROR);
 			}
+			if (dev_key == 0) {
+				Tcl_SetResult(interp, "Can't close until after "
+				    "device has been opened", TCL_STATIC);
+				return (TCL_ERROR);
+			}
+			tclm_interp->DeleteDevice(dev_key);
 		} else if (strcmp(sub_argv[0], "midithru") == 0) {
 			switch (sub_argc) {
 			case 1:
@@ -352,10 +357,98 @@ Tclm_MidiDevice(ClientData client_data, Tcl_Interp *interp, int argc,
 			default:
 				/* wrong args */
 				Tcl_AppendResult(interp, "Wrong number of "
-				    "args: should be \"", argv[0], " midithru "
-				    "?on|off?\"", 0);
+				    "args: should be \"", argv[0], " DevID "
+				     "midithru ?on|off?\"", 0);
 				return (TCL_ERROR);
 			}
+		} else if (strcmp(sub_argv[0], "slave") == 0) {
+			MidiDevice *master;
+			if (sub_argc != 2) {
+				Tcl_AppendResult(interp, "Wrong number of "
+				    "args: should be \"", argv[0], " DevID "
+				    "slave DevID\"", 0);
+				return (TCL_ERROR);
+			}
+			master = tclm_interp->GetDevice(sub_argv[1]);
+			if (master == 0) {
+				Tcl_AppendResult(interp, "Bad Master ID \"",
+				    sub_argv[1], "\"", 0);
+				return (TCL_ERROR);
+			}
+			if (!dev->Slave(*master)) {
+				Tcl_AppendResult(interp, "Couldn't make "
+				    "slave: ", dev->GetError(), 0);
+				return (TCL_ERROR);
+			}
+		} else if (strcmp(sub_argv[0], "channel_on") == 0 ||
+		    strcmp(sub_argv[0], "channel_off") == 0) {
+			if (strcmp(sub_argv[0], "channel_on") == 0)
+				con = 1;
+			else
+				con = 0;
+
+			switch (sub_argc) {
+			case 1:
+				/* return the value */
+				buf = new ostrstream;
+
+				imask = dev->GetChannelMask();
+				if (imask == -1) {
+					Tcl_SetResult(interp,
+					    (char *)dev->GetError(),
+					    TCL_VOLATILE);
+					return (TCL_ERROR);
+				}
+				mask = imask;
+				if (con)
+					*buf << "channel_on ";
+				else
+					*buf << "channel_off ";
+				for (j = 0; j < 16; j++) {
+					if (!(mask & (1 << j))) {
+						if (con)
+							*buf << " " << j;
+					} else {
+						if (!con)
+							*buf << " " << j;
+					}
+				}
+				*buf << ends;
+				str = buf->str();
+				Tcl_AppendElement(interp, str);
+				delete str;
+				delete buf;
+				break;
+			default:
+				/* get the current mask */
+				imask = dev->GetChannelMask();
+				if (imask == -1) {
+					Tcl_SetResult(interp,
+					    (char *)dev->GetError(),
+					    TCL_VOLATILE);
+					return (TCL_ERROR);
+				}
+				mask = imask;
+
+				/* set the value */
+				for (j = 1; j < sub_argc; j++) {
+					if (Tcl_GetInt(interp, sub_argv[j],
+					    &value) != TCL_OK)
+						return (TCL_ERROR);
+					if (con) {
+						mask &= ~(1 << value);
+					} else {
+						mask |= (1 << value);
+					}
+				}
+				if (!dev->SetChannelMask(mask)) {
+					Tcl_SetResult(interp,
+					    (char *)dev->GetError(),
+					    TCL_VOLATILE);
+					return (TCL_ERROR);
+				}
+				break;
+			}
 		} else {
 			Tcl_AppendResult(interp, "Bad parameter \"",
 			    sub_argv[0], "\"", 0);
@@ -367,79 +460,182 @@ Tclm_MidiDevice(ClientData client_data, Tcl_Interp *interp, int argc,
 }
 
 int
-#ifdef _WINDOWS
-FAR PASCAL __loadds
-#endif
 Tclm_MidiTime(ClientData client_data, Tcl_Interp *interp, int argc,
     char **argv)
 {
 	ostrstream tbuf;
-	TimeType tt;
+	unsigned long t;
 	TclmInterp *tclm_interp;
 	MidiDevice *dev;
 	char *str;
 
-	if (argc < 2 || argc > 3) {
+	if (argc != 2) {
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " {smf|smpte} ?value?\"", 0);
+		    argv[0], " DevID\"", 0);
 		return (TCL_ERROR);
 	}
 	tclm_interp = (TclmInterp *)client_data;
-	dev = tclm_interp->GetMidiDevice();
+	dev = tclm_interp->GetDevice(argv[1]);
 	if (dev == 0) {
-		Tcl_SetResult(interp, "0", TCL_STATIC);
-		return (TCL_OK);
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
 	}
-	if (strcmp(argv[1], "smf") == 0)
-		tt = SMFTIME;
-	else if (strcmp(argv[1], "smpte") == 0)
-		tt = SMPTETIME;
-	else {
-		Tcl_AppendResult(interp, "bad time type \"", argv[1],
-		    "\" must be smf or smpte", 0);
+	if (!dev->GetTime(&t)) {
+		Tcl_AppendResult(interp, "Couldn't get time: ",
+		    dev->GetError() , 0);
+		return (TCL_ERROR);
+	}
+	tbuf << t << ends;
+	str = tbuf.str();
+	Tcl_SetResult(interp, str, TCL_VOLATILE);
+	delete str;
+	return (TCL_OK);
+}
+
+int
+Tclm_MidiFeature(ClientData client_data, Tcl_Interp *interp, int argc,
+    char **argv)
+{
+	TclmInterp *tclm_interp;
+	MidiDevice *dev;
+	char **sub_argv;
+	int i, sub_argc;
+
+	if (argc < 3) {
+		Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    argv[0], " DevID ?kernel_timing? ?smpte_timing? "
+		    "?mpu401_timing? ?get_smpte?\"", 0);
+		return (TCL_ERROR);
+	}
+	tclm_interp = (TclmInterp *)client_data;
+	dev = tclm_interp->GetDevice(argv[1]);
+	if (dev == 0) {
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
 		return (TCL_ERROR);
 	}
 
-	if (argc == 3) {
-		Tcl_SetResult(interp, "Setting time not currently supported",
-		    0);
-		return (TCL_ERROR);
-	} else {
-		GetTime(tbuf, dev, tt);
-		str = tbuf.str();
-		Tcl_SetResult(interp, str, TCL_VOLATILE);
-		delete str;
+	for (i = 2; i < argc; i++) {
+		ostrstream *res;
+		char *res_str;
+
+		// loop through each arg and either set or return values
+		if (Tcl_SplitList(interp, argv[i], &sub_argc, &sub_argv)
+		    != TCL_OK)
+			return (TCL_ERROR);
+		if ((res = dev->Feature(sub_argv[0],
+		    (const char **)&sub_argv[1], sub_argc - 1)) == 0) {
+			Tcl_AppendResult(interp, "Feature \"", sub_argv[0],
+			    "\" failed: ", dev->GetError(), 0);
+			return (TCL_ERROR);
+		}
+		res_str = res->str();
+		Tcl_AppendResult(interp, res_str, 0);
+		delete res_str;
+		delete res;
 	}
 	return (TCL_OK);
 }
 
-void
-GetTime(ostrstream &buf, MidiDevice *dev, TimeType type)
+int
+Tclm_MidiSend(ClientData client_data, Tcl_Interp *interp, int argc,
+    char **argv)
 {
-	SMPTETime smpte_t;
-	unsigned long smf_t;
+	TclmInterp *tclm_interp;
+	MidiDevice *dev;
+	Event **events;
+	int i, num_events;
 
-	switch (type) {
-	case SMFTIME:
-		if (!dev->GetTime(SMFTIME, &smf_t))
-			buf << "ERR";
-		else
-			buf << smf_t;
-		buf << ends;
-		break;
-	case SMPTETIME:
-		if (!dev->GetTime(SMPTETIME, &smpte_t))
-			buf << "ERR";
-		else {
-			if (!(smpte_t.GetFlags() & SMPTE_INSYNC))
-				buf << "NOSYNC";
-			else
-				buf << smpte_t;
-		}
-		buf << ends;
-		break;
-	default:
-		buf << "Bad TimeType" << ends;
-		break;
+	if (argc < 3) {
+		Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    argv[0], " DevID event ?event ...?\"", 0);
+		return (TCL_ERROR);
 	}
+	tclm_interp = (TclmInterp *)client_data;
+
+	dev = tclm_interp->GetDevice(argv[1]);
+	if (dev == 0) {
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
+	}
+
+	events = new Event *[argc - 2];
+	if (events == 0) {
+		Tcl_SetResult(interp, "Out of memory", TCL_STATIC);
+		return (TCL_ERROR);
+	}
+	num_events = 0;
+	for (i = 2; i < argc; i++) {
+		events[num_events] = Tclm_ParseEvent(interp, argv[i]);
+		if (events[num_events] != 0) {
+			if (events[num_events]->GetType() == NOTEON &&
+			    ((NoteEvent *)events[num_events])->GetNotePair()
+			    != 0) {
+				Tcl_AppendResult(interp, "Can't send Note "
+				    "events, use separate NoteOn/NoteOff "
+				    "events", 0);
+				delete ((NoteEvent *)events[num_events])
+				    ->GetNotePair();
+				delete events[num_events];
+			} else {
+				num_events++;
+			}
+		} else {
+			if (strlen(interp->result) == 0) {
+				Tcl_SetResult(interp, "Out of memory",
+				    TCL_STATIC);
+				return (TCL_ERROR);
+			}
+		}
+	}
+	if (!dev->Send(events, num_events)) {
+		Tcl_SetResult(interp, (char *)dev->GetError(), TCL_VOLATILE);
+		return (TCL_ERROR);
+	}
+	for (i = 0; i < num_events; i++)
+		delete events[i];
+	delete events;
+	return (TCL_OK);
+}
+
+int
+Tclm_MidiRecv(ClientData client_data, Tcl_Interp *interp, int argc,
+    char **argv)
+{
+	ostrstream *buf;
+	TclmInterp *tclm_interp;
+	MidiDevice *dev;
+	Event **events;
+	int i, num_events;
+	char *str;
+
+	if (argc != 2) {
+		Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    argv[0], " DevID\"", 0);
+		return (TCL_ERROR);
+	}
+	tclm_interp = (TclmInterp *)client_data;
+
+	dev = tclm_interp->GetDevice(argv[1]);
+	if (dev == 0) {
+		Tcl_AppendResult(interp, "Bad DevID \"", argv[1], "\"", 0);
+		return (TCL_ERROR);
+	}
+
+	if (!dev->Recv(&events, &num_events)) {
+		Tcl_SetResult(interp, (char *)dev->GetError(), TCL_VOLATILE);
+		return (TCL_ERROR);
+	}
+
+	for (i = 0; i < num_events; i++) {
+		buf = new ostrstream;
+		Tclm_PrintEvent(*buf, events[i]);
+		str = buf->str();
+		if (str != 0 && str[0] != '\0')
+			Tcl_AppendElement(interp, str);
+		delete str;
+		delete buf;
+		delete events[i];
+	}
+	delete events;
+	return (TCL_OK);
 }
